require "auth/service/authenticator/version"
require 'yaml'
require "json"
require 'net/http'

module Auth
  module Service
    module Authenticator

      class Manager

        @@auth_url = ''
        @@auth_port = ''

        # Authenticate user
        def self.authenticate(email, password)
          self.configure
          uri = URI.parse("#{@@auth_url}")
          http = Net::HTTP.new(@@auth_url, @@auth_port)
          request = Net::HTTP::Post.new("/api/v1/authenticate")
          request.add_field('Content-Type', 'application/json')
          request.body = {'email': email, 'password': password}.to_json
          response = http.request(request)

          result = case response
            when Net::HTTPSuccess then
              { user: JSON.parse(response.body), status: response.code, message: '' }
            when Net::HTTPClientError then
              { error: 'usuario ou senha inválidos.', status: response.code, message: response.message }
            when Net::ReadTimeout then
              { error: 'timeout', status: response.code, message: response.message }
            else
              { error: 'erro inesperado. tente novamente mais tarde.', status: '422', message: '' }
            end
          return result
        end

        def self.validate_token(token)
          uri = URI.parse("#{@@auth_url}")
          http = Net::HTTP.new(@@auth_url, @@auth_port)
          request = Net::HTTP::Post.new("/api/v1/validate")
          request.add_field('Content-Type', 'application/json')
          request.add_field('Authorization', token)
          response = http.request(request)

          result = case response
            when Net::HTTPSuccess then
              { user: JSON.parse(response.body), status: response.code, message: '' }
            when Net::HTTPClientError then
              { error: 'token invalido.', status: response.code, message: response.message }
            when Net::ReadTimeout then
              { error: 'timeout', status: response.code, message: response.message }
            else
              { error: 'erro inesperado. tente novamente mais tarde.', status: '422', message: '' }
            end
          return result
        end

        def self.logout(token)
          uri = URI.parse("#{@@auth_url}")
          http = Net::HTTP.new(@@auth_url, @@auth_port)
          request = Net::HTTP::Delete.new("/api/v1/authenticate")
          request.add_field('Content-Type', 'application/json')
          request.add_field('Authorization', token)
          response = http.request(request)

          result = case response
            when Net::HTTPNoContent then
              { status: response.code, message: 'success', success: true }
            when Net::HTTPClientError then
              { error: 'Token invalido.', status: response.code, message: response.message }
            when Net::ReadTimeout then
              { error: 'timeout', status: response.code, message: response.message }
            else
              { error: 'erro inesperado. tente novamente mais tarde.', status: '422', message: '' }
            end
          return result
        end

        private
          # Setup auth server configuration
          def self.configure
            if @@auth_url == ''
              begin
                # Load app configuration for server URI in you rails app
                config = YAML.load_file('config/authenticator_config.yml')
                @@auth_url = config['url']
                @@auth_port = config['port']
              rescue
                # default configuration
                @@auth_url = 'localhost'
                @@auth_port = '4000'
              end
            end
          end
      end

    end
  end
end
